/***************************************************************************
*    Copyright 2001  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: krbTasks.c
*    Creation Date: 19 Sept. 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.4 $
*        $Date: 2005/08/15 15:02:09 $
*
****************************************************************************
*    Description:
*       Implementation of Wake UP, rekey and Time Manager tasks for security
*
*
****************************************************************************/
//#define WIN32XCH 1

//#ifdef WIN32XCH
//#include "krb.h"
//#else
//#include <pna.h>
//#include <rescfg.h>
//#endif

#include <time.h>
#include <hausenv.h>
///#include <xbcmos.h>
#include <net.h>
#include "seckrb.h"
#include <krb5client.h>
#include <appos.h>
#include <globalCfg.h>
#include "krbTasks.h"
#include "serverAuthKey.h"
#include "rekeyReplay.h"
#include "timeTable.h"

#include "bcmosHelper.h"

#define KRBM_TASK_STACK         12*1024
#define KRBM_TASK_PRIORITY       OS_TASK_DATA
#define KRBM_MAX_DGRAM_SIZE      1024
#define WKUP_KEY_MGMNT_MSGID     0x01
#define REKEY_KEY_MGMNT_MSGID    0x05




static   SEC_PROV_INFO_BLOCK     secProvInfoBlock;
static   unsigned char           wk_pkt[KRBM_MAX_DGRAM_SIZE];
int      wakeup_socket;
#if CH_NEEDED

extern int  aprepParseCmsASD(CMS_ASD_APREP *pCmsAsd, char *msg);
//extern int  validateIPSECCiphersuites(int *enc, int *auth);
extern void prfGetSecInfoBlock(SEC_PROV_INFO_BLOCK *secProvInfoBlock);



/*****************************************************************************
** FUNCTION:  krbProcessWakeUpMessage
**
** PURPOSE:   process a wake up message
**
** PARAMETERS:  wk_pkt - wake up message payload
**              wk_pkt_len - length of the messsage
**
** RETURNS:
*****************************************************************************/
static void krbProcessWakeUpMessage(unsigned char *wk_pkt, int wk_pkt_len)
{
   int                    ix = 0;
   int                    i;
   int                    temp;
   int                    doi;
   int                    rc=33;
   int                    server_nonce;
   unsigned char          krb_server_name[255];
   BOOL                   cmsFound = FALSE;
   KRBCLI_AUTH_BLOCK      krbCliBlock;
   PROV_CLIENT_KEYS       provClientKeys;
   PROV_ASD_APREQ         provAsdReq;
   PROV_ASD_APREP         provAsdRep;
   CMS_CLIENT_KEYS        cmsClientKeys;
   CMS_ASD_APREQ          cmsAsdReq;
   CMS_ASD_APREP          cmsAsdRep;
   char                   *p;


   LOG(("Processing Wake-up"));
   /* key management message ID */
   if ( wk_pkt[ix++] != WKUP_KEY_MGMNT_MSGID )
      LOG(("KRB SA Maintain Task: Bad key management message ID."));

   LOG(("Processing DOI"));
   /* DOI */
   doi = wk_pkt[ix++];
   switch ( doi )
   {
      //case KRB_IPSEC_DOI:
      //   LOG(("KRB SA Maintain Task: DOI is IPSEC"));
      //   break;
      case KRB_SNMPV3_DOI:
         LOG(("KRB SA Maintain Task: DOI is SNMPV3"));
         break;
      default:
         LOG(("KRB SA Maintain Task: ERROR in DOI"));
         return;
         break;
   }

   LOG(("Processing protocol version"));
   /* protocol version */
   if ( wk_pkt[ix++] != 0x10 )
      LOG(("KRB SA Maintain Task: Bad protocol version."));

   LOG(("Processing server_nonce"));
   /* server_nonce */
#if (CPU_LITTLE_ENDIAN != 0)
   memcpy(&temp, &wk_pkt[ix], 4);
   server_nonce = ntohl(temp);
   ix += 4;
#else
   memcpy(&server_nonce, &wk_pkt[ix], 4);
   ix += 4;
#endif
   LOG(("KRB SA Maintain Task: server_nonce=%d", server_nonce));

   temp = wk_pkt_len - ix;
   /* server kerberos principal name */
   memcpy(krb_server_name, &wk_pkt[ix], temp);
   krb_server_name[temp] = '\0';
   LOG(("KRB SA Maintain Task: krb_Server_name=%s", krb_server_name));

   /* fqdn is case insensitive */
   p = krb_server_name;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   switch ( doi )
   {
      //case KRB_IPSEC_DOI:
      //   //for ( i=0;((i<secProvInfoBlock.totEndPoints)&&(!cmsFound));i++)
      //   for ( i=0;((i<2)&&(!cmsFound));i++)
      //   {
      //      secProvInfoBlock.CmsAuthBlockCB(i, &krbCliBlock);
      //      if ( ! memcmp(krbCliBlock.szServiceName, krb_server_name,
      //               strlen((char*)krb_server_name)))
      //      {
      //         secProvInfoBlock.CmsASDReqCB(i, &cmsAsdReq);
      //         LOG(("Found endPoint provisioned with Call Agent."));
      //         cmsFound = TRUE;
      //      }
      //   }
      //   /* override the server_nonce */
      //   krbCliBlock.serverNonce = server_nonce;
      //   krbCliBlock.krbReason = KRB_WAKEUP;
      //
      //   rc = secKrb5ClientAuthenticate(&krbCliBlock,
      //         (void*)&cmsAsdReq,
      //         (void*)&cmsAsdRep,
      //         (void*)&cmsClientKeys);
       //  break;
      case KRB_SNMPV3_DOI:
         LOG(("KRB SA Maintain Task: DOI is SNMPV3"));
         secProvInfoBlock.ProvAuthBlockCB(&krbCliBlock);
         secProvInfoBlock.ProvASDReqCB(&provAsdReq);
         /* override the server_nonce */
         krbCliBlock.serverNonce = server_nonce;
         krbCliBlock.krbReason = KRB_WAKEUP;

         rc = secKrb5ClientAuthenticate(&krbCliBlock,
               (void*)&provAsdReq,
               (void*)&provAsdRep,
               (void*)&provClientKeys);

         break;
      default:
         break;
   }

   if (rc)
      printf("KRB SA Maintain Task: secKrb5ClientAuthenticate FAILED.\n");
   else
      printf("KRB SA Maintain Task: secKrb5ClientAuthenticate SUCCEEDED.\n");
}


/*****************************************************************************
** FUNCTION:  krbProcessRekeyMessage
**
** PURPOSE:   process a Rekey message
**
** PARAMETERS:  rekey_pkt - Rekey message payload
**              rekey_pkt_len - length of the messsage
**
** RETURNS:
*****************************************************************************/
static void krbProcessRekeyMessage(unsigned char *rekey_pkt, int rekey_pkt_len)
{
   int                    ix = 0;
   int                    i;
   int                    temp;
   int                    doi;
   int                    rc;
   int                    server_nonce;
   unsigned char          krb_server_name[255];
   unsigned char          utc_time[32];
   BOOL                   cmsFound = FALSE;
   KRBCLI_AUTH_BLOCK      krbCliBlock;
   CMS_CLIENT_KEYS        cmsClientKeys;
   CMS_ASD_APREQ          cmsAsdReq;
   CMS_ASD_APREP          pCmsAsd;
   unsigned long          splt;
   unsigned long          gp;
   int                    cs_len;
   int                    enc[MAX_AP_CSUITES_LENGTH] = {0};
   int                    auth[MAX_AP_CSUITES_LENGTH] = {0};
   unsigned long          re_establish_flag;
   unsigned char          server_auth_key[32];
   unsigned char          sig[20];
   unsigned char          sig1[20];
   int                    status;
   int                    tmp;
   int                    clock_skew;
   int                    tmp_time;
   int                    utc_time_offset;
   unsigned long          auth_key_len;
   unsigned char          mta_utc_str[16];
   unsigned long          mta_utc_len;
   unsigned long          mta_time;
   unsigned long          ca_time;
   unsigned long          tmdiff;
   char                   *p;



   LOG(("Processing Rekey"));
   /* key management message ID */
   if ( rekey_pkt[ix++] != REKEY_KEY_MGMNT_MSGID )
      LOG(("KRB SA Maintain Task: Bad key management message ID."));

   LOG(("Processing DOI"));
   /* DOI */
   doi = rekey_pkt[ix++];
   switch ( doi )
   {
     // case KRB_IPSEC_DOI:
     //    LOG(("KRB SA Maintain Task: DOI is IPSEC"));
     //    break;
      default:
         LOG(("KRB SA Maintain Task: ERROR in DOI"));
         return;
         break;
   }

   LOG(("Processing protocol version"));
   /* protocol version */
   if ( rekey_pkt[ix++] != 0x10 )
      LOG(("KRB SA Maintain Task: Bad protocol version."));

   LOG(("Processing server_nonce"));
   /* server_nonce */
#if (CPU_LITTLE_ENDIAN != 0)
   memcpy(&temp, &rekey_pkt[ix], 4);
   server_nonce = ntohl(temp);
   ix += 4;
#else
   memcpy(&server_nonce, &rekey_pkt[ix], 4);
   ix += 4;
#endif
   LOG(("KRB SA Maintain Task: server_nonce=%d", server_nonce));

   /* server kerberos principal name  (null terminated)*/
   strcpy(krb_server_name, &wk_pkt[ix]);
   LOG(("KRB SA Maintain Task: krb_Server_name=%s", krb_server_name));
   ix += strlen(krb_server_name)+1;

   /* fqdn is case insensitive */
   p = krb_server_name;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   /* verify the signature here */
   status = secGetServerAuthKey(krb_server_name, server_auth_key,
         &auth_key_len);

   printf("Server Auth Key is ");
   for (i=0;i<20;i++ )
      printf("%x ", server_auth_key[i]);
   printf("\n");
   if (status == SERVER_AUTH_KEY_NOT_FOUND )
   {
      LOG(("FOR %s SERVER_AUTH_KEY_NOT_FOUND.", krb_server_name));
      return;
   }
   SecLibGetCryptoIface()->secCryptoCalcHash(
         HASH_SHA1_TYPE, rekey_pkt, rekey_pkt_len-20, server_auth_key,
         auth_key_len, sig, &temp);

   temp = rekey_pkt_len - 20;

   memcpy(sig1, &rekey_pkt[temp],20);
   if ( memcmp(sig,sig1,20))
   {
      printf("REKEY SIGNATURE Did not Verify.\n");
      return;
   }

   /* 13 bytes of UTC time (not null terminated) */
   memcpy(utc_time, &rekey_pkt[ix], 13);
   utc_time[13] = '\0';
   LOG(("KRB SA Maintain Task: UTC Time=%s", utc_time));
   ix += 13;

   if ( utc_time[12] != 'Z' )
   {
      LOG(("KRB SA Maintain Task UTC FORMAT ERROR.\n"));
      return;
   }


   SecLibGetCryptoIface()->secCryptoConvertUTCToTime(utc_time, 13, &ca_time);

   /* Check to see if this is a replay */
   if ( secCheckRekeyReplay(server_nonce,
            krb_server_name,
            ca_time) == REKEY_REPLAY_FOUND )

   {
      LOG(("KRB SA Maintain Task: REKEY is a replay."));
      return;
   }
   else
   {
      secStoreRekeyMessage(server_nonce,
            krb_server_name,
            ca_time);
   }

   /* calculate MTA and CMS times in seconds in UTC */
#ifdef DEMO
   utc_time_offset = 0;
#else
   utc_time_offset = cmMiscGetUTCTimeOffset();
#endif
   LOG(("CM UTC Time Offset is %d.", utc_time_offset));
   tmp_time -= utc_time_offset;
   SecLibGetCryptoIface()->secCryptoGetCurrUTCTime(tmp_time, mta_utc_str, &mta_utc_len);
   LOG(("MTA's UTC time is %s", mta_utc_str));
   SecLibGetCryptoIface()->secCryptoConvertUTCToTime(mta_utc_str, mta_utc_len, &mta_time);

   /* get the time skew for this CMS */
   clock_skew =  SecLibGetNVRAMIface()->secReadNVRAMSkew(AP_CMS_FLAG, krb_server_name,
         strlen(krb_server_name));
   mta_time -= clock_skew;

   tmdiff = labs(mta_time - ca_time);
   if ( tmdiff > MAX_ALLOWED_CLOCK_SKEW_IN_SECONDS )
   {
      LOG(("CA's Time %d, MTA's time %d", ca_time, mta_time));
      LOG(("KRB SA Maintain Task: ERROR TIME SKEW(%d) GREATER THAN MAX(%d)",
               tmdiff,MAX_ALLOWED_CLOCK_SKEW_IN_SECONDS));
   }
   else
   {
      LOG(("KRB SA Maintain Task TIME DIFFERENCE IS %d seconds.",tmdiff));
   }

   /* ASD for IPSEC */
   ix += aprepParseCmsASD(&pCmsAsd, &rekey_pkt[ix]);
   LOG(("ASD SPI value = %d", pCmsAsd.inbound_SPI));

   /* length of ciphersuit list */
   cs_len = rekey_pkt[ix++];

   /* list of ciphersuites */
   for (i=0;i<cs_len;i++)
   {
      auth[i] = rekey_pkt[ix++];
      enc[i] = rekey_pkt[ix++];
   }

   /* for now, just return the first ciphersuites in the list */
   if ( validateIPSECCiphersuites(enc, auth) != 0 )
   {
      LOG(("IPSEC AP ciphersuites not valid.\n"));
      return;
   }

#if (CPU_LITTLE_ENDIAN != 0)
   temp = 0;
   memcpy(&temp, &rekey_pkt[ix], 4);
   splt = ntohl(temp);
   ix += 4;
   memcpy(&temp, &rekey_pkt[ix], 4);
   gp = ntohl(temp);
   ix += 4;
#else
   memcpy(&splt, &rekey_pkt[ix], 4);
   ix += 4;
   memcpy(&gp, &rekey_pkt[ix], 4);
   ix += 4;
#endif

   printf("Security Parameter Lifetime = %d seconds.\n", splt);
   printf("Grace Period = %d seconds.\n", gp);

   re_establish_flag = rekey_pkt[ix++];
   printf("Reestablish Flag 0x%x.\n", re_establish_flag);

   for ( i=0;((i<secProvInfoBlock.totEndPoints)&&(!cmsFound));i++)
   {
      secProvInfoBlock.CmsAuthBlockCB(i, &krbCliBlock);
      if ( ! memcmp(krbCliBlock.szServiceName, krb_server_name,
               strlen(krb_server_name)))
      {
         secProvInfoBlock.CmsASDReqCB(i, &cmsAsdReq);
         LOG(("Found endPoint provisioned with Call Agent."));
         cmsFound = TRUE;
      }
   }
   if (i==secProvInfoBlock.totEndPoints)
   {
      LOG(("KRB SA Maintain Task: NO ENDPOINT PROVISIONED TO THIS CALL AGENT"));
      return;
   }

   /* override the server_nonce */
   krbCliBlock.serverNonce = server_nonce;
   krbCliBlock.krbReason = KRB_REKEY;

   /* pass the rekey info gathered from the rekey message */
   krbCliBlock.rekeyInfoBlock.authTID = auth[0];
   krbCliBlock.rekeyInfoBlock.encTID = enc[0];
   krbCliBlock.rekeyInfoBlock.secParamLifeTime = splt;
   krbCliBlock.rekeyInfoBlock.gracePeriod = gp;

   rc = secKrb5ClientAuthenticate(&krbCliBlock,
         (void*)&cmsAsdReq,
         (void*)&pCmsAsd,
         (void*)&cmsClientKeys);

   if (rc)
      printf("KRB SA Maintain Task: secKrb5ClientAuthenticate FAILED.\n");
   else
      printf("KRB SA Maintain Task: secKrb5ClientAuthenticate SUCCEEDED.\n");
}

/*****************************************************************************
** FUNCTION:  krbSAMaintainInitCB
**
** PURPOSE:   Init for wake up task
**
** PARAMETERS:
**
**
** RETURNS:   Success or failure
*****************************************************************************/
static void krbSAMaintainInitCB(void)
{
   int                bufsize = KRBM_MAX_DGRAM_SIZE;

   LOG(("krbSAMaintainInitCB."));
   LOG(("KRB SA Maintain Task: Init" ));
   wakeup_socket = 0;
   /* Open the socket */
   if ( bcmNetUdpOpen( (UINT16)AP_UDP_PORT_NUMBER, TRUE, &wakeup_socket ) != NET_STATUS_OK )
   {
      LOG(("KRB SA Maintain Task Init: could not open socket" ));
      return ;
   }
   else
   {
      LOG(("KRB SA Maintain Task Init: Socket opened successfully wakeup_socket=%d.", wakeup_socket));
   }
    printf("stuff removed x\n");
   /* increase size of sending buffer */
 //trc  if (setsockopt(wakeup_socket, SOL_SOCKET, SO_SNDBUF, (char *) &bufsize,
 //           sizeof(bufsize)) )
 //  {
 //     LOG(("could not set sendbuf size, errno = 0x%x", errno));
 //  }
 //
 //  /* increase size of receiving buffer */
 //  if (setsockopt(wakeup_socket, SOL_SOCKET, SO_RCVBUF, (char *) &bufsize,
 //           sizeof(bufsize)) )
 //  {
 //     LOG(("could not set recvbuf size, errno = 0x%x", errno));
 //  }


}
/*****************************************************************************
** FUNCTION:  krbSAMaintainDeinitCB
**
** PURPOSE:   Deinit for wake up task
**
** PARAMETERS:
**
**
** RETURNS:   Success or failure
*****************************************************************************/
static void krbSAMaintainDeinitCB(void)
{
         /* close socket */
         bcmNetUdpClose( wakeup_socket );

}

OS_TASKID krbSAMaintainTaskID;
/*****************************************************************************
** FUNCTION:  krbStartTasks
**
** PURPOSE:   Create and call the threads for wake up and time management
**
** PARAMETERS:
**
**
** RETURNS:   Success or failure
*****************************************************************************/
int krbStartTasks(void)
{
//   OS_STATUS status;


#if 0
   /* start task to handle AP Wake up messages */
   status = bcmOsTaskCreate("KRBM", KRBM_TASK_STACK, KRBM_TASK_PRIORITY,
         krbSAMaintainTaskMain, 0, &krbSAMaintainTaskID);
   if (status != OS_STATUS_OK)
   {
      LOG(("ERROR krbStartTasks: Unable to create kerberos wake up task"));
      return status;
   }
#endif

printf("removed stuff ss\n");
//trc
//#if 1
//   /* start task to handle AP Wake up messages */
//   status = appOsTaskCreate(
//         "KRBM",
//         KRBM_TASK_STACK,
//         KRBM_TASK_PRIORITY,
//         (void*)krbSAMaintainInitCB,
//         (void*)krbSAMaintainTaskMain,
//         (void*)krbSAMaintainDeinitCB,
//         0,
//         &krbSAMaintainTaskID);
//   if (status != OS_STATUS_OK)
//   {
//      LOG(("ERROR krbStartTasks: Unable to create kerberos wake up task"));
//      return status;
//   }
//   else
//     LOG(("krbStartTasks: Created KRBM successfully"));
//#endif

#ifndef DEMO
   /* initialize the sec prov info block */
   prfGetSecInfoBlock(&secProvInfoBlock);
#endif

   return 0;
}
 /*****************************************************************************
** FUNCTION:  krbShutDownTasks
**
** PURPOSE:   Shutdown the tasks**
** PARAMETERS:
**
**
** RETURNS:   Success or failure
*****************************************************************************/
#if 0
int krbShutDownTasks (void)
{
   /* Currently, the krb task should be blocked in the reset-acknowledgement state.
    * Advance the task to the reset-active state. This will cause the task to
    * resume operation. In this state, the task should perform any operations
    * required for proper shutdown before exiting from it's main task function. */
   appOsTaskProceedToShutdown( krbSAMaintainTaskID );

   /* Wait for the krb task to complete de-initialization */
   appOsTaskWaitForDone( krbSAMaintainTaskID );
  return 0; //trc
}
#endif
#endif //ch_needed
/*
******************************************************************************
** FUNCTION:  reAuthenticate
**
** PURPOSE:   renew any ticket or subkey if expired
**
** PARAMETERS:   flag - AS or AP, prov or CMS
**               fqdn - only in case of CMS
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void reAuthenticate(int flag, unsigned char *fqdn)
{
   int                    rc;
   int                    i;
   BOOL                   cmsFound = FALSE;
   KRBCLI_AUTH_BLOCK      krbCliBlock;
   PROV_CLIENT_KEYS       provClientKeys;
   PROV_ASD_APREQ         provAsdReq;
   PROV_ASD_APREP         provAsdRep;
   CMS_CLIENT_KEYS        cmsClientKeys;
   CMS_ASD_APREQ          cmsAsdReq;
   CMS_ASD_APREP          cmsAsdRep;
   char                   tmp_fqdn[255];
   char                   tmp_sname[255];
   char                   *p;

   /* case insensitive */
   strcpy(tmp_fqdn, fqdn);
   p = tmp_fqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   switch (flag)
   {
      case AS_PROV_FLAG:
         /* do AS req rep for snmpv3 */
         secProvInfoBlock.ProvAuthBlockCB(&krbCliBlock);
         secProvInfoBlock.ProvASDReqCB(&provAsdReq);
         krbCliBlock.krbReason = KRB_TICKET_EXPIRED;

         rc = secKrb5ClientAuthenticate(&krbCliBlock,
               (void*)&provAsdReq,
               (void*)&provAsdRep,
               (void*)&provClientKeys);
         break;

      case AS_CMS_FLAG:

 //        /* do AS  req rep for IPSEC */
 //
 //        //for ( i=0;((i<secProvInfoBlock.totEndPoints)&&(!cmsFound));i++)
 //        for ( i=0;((i<2)&&(!cmsFound));i++)
 //        {
 //           secProvInfoBlock.CmsAuthBlockCB(i, &krbCliBlock);
 //           /* case insensitive */
 //           strcpy(tmp_sname, krbCliBlock.szServiceName);
 //           p = tmp_sname;
 //           while(*p)
 //           {
 //              *p++ = (char)tolower((int)*p);
 //           }
 //           if ( ! memcmp(tmp_sname, tmp_fqdn, strlen(tmp_fqdn)))
 //           {
 //              secProvInfoBlock.CmsASDReqCB(i, &cmsAsdReq);
 //              LOG(("Found endPoint provisioned with Call Agent."));
 //              cmsFound = TRUE;
 //           }
 //        }
 //        /* if this cms is not provisioned to any endpoit, nothing to do */
 //        if ( cmsFound == FALSE )
 //        {
 //           LOG(("Renew CMS TKT: Nothing to do. No endpoit provisioned to cms %s.", fqdn));
 //           return;
 //        }
 //        krbCliBlock.krbReason = KRB_TICKET_EXPIRED;
 //
 //        rc = secKrb5ClientAuthenticate(&krbCliBlock,
 //              (void*)&cmsAsdReq,
 //              (void*)&cmsAsdRep,
 //              (void*)&cmsClientKeys);
 //        break;
//
//      case AP_CMS_FLAG:
//         /* do AP req rep for IPSEC */
//         // for ( i=0;((i<secProvInfoBlock.totEndPoints)&&(!cmsFound));i++)
//         for ( i=0;((i<2)&&(!cmsFound));i++)
//         {
//            secProvInfoBlock.CmsAuthBlockCB(i, &krbCliBlock);
//            /* case insensitive */
//            strcpy(tmp_sname, krbCliBlock.szServiceName);
//            p = tmp_sname;
//            while(*p)
//            {
//               *p++ = (char)tolower((int)*p);
//            }
//            if ( ! memcmp(tmp_sname, tmp_fqdn, strlen(tmp_fqdn)))
//            {
//               secProvInfoBlock.CmsASDReqCB(i, &cmsAsdReq);
//               LOG(("Found endPoint provisioned with Call Agent."));
//               cmsFound = TRUE;
//            }
//         }
//
//         /* if this cms is not provisioned to any endpoit, nothing to do */
//         if ( cmsFound == FALSE )
//         {
//            LOG(("Renew CMS Subkey: Nothing to do. No endpoit provisioned to cms %s.", fqdn));
//            return;
//         }
//
//         krbCliBlock.krbReason = KRB_SUBKEY_EXPIRED;
//
//         rc = secKrb5ClientAuthenticate(&krbCliBlock,
//               (void*)&cmsAsdReq,
//               (void*)&cmsAsdRep,
//               (void*)&cmsClientKeys);
      default:
         break;
   }
}
  

