/***************************************************************************
 *     Copyright 2001  Broadcom Corporation
 *     All Rights Reserved
 *     No portions of this material may be reproduced in any form without the
 *     written permission of:
 *              Broadcom Corporation
 *              16251 Laguna Canyon Road
 *              Irvine, California  92618
 *     All information contained in this document is Broadcom Corporation
 *     company private, proprietary, and trade secret.
 *
 ****************************************************************************
 *
 *     Filename:  serverAuthKey.c
 *     Creation Date: 30 October 2001 (v0.00)
 *     VSS Info:
 *         $Revision: 1.1 $
 *         $Date: 2002/08/21 17:54:11 $
 *
 ****************************************************************************
 *     Description:
 *
 *       Implementation of server auth key
 *
 *     Notes:
 *
 ****************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "serverAuthKey.h"

#ifdef DEMO
#define GLOBAL_CFG_MAX_ENDPT   8
#else
#include <globalCfg.h>
#endif


#define MAX_KEY_STORAGE_ENTRIES    GLOBAL_CFG_MAX_ENDPT
#define SERVER_AUTH_KEY_LEN        SHA1_SIZE

/* structs using in table to map info to NVRAM */
typedef struct
{
   unsigned char   t_hash[MD5_SIZE];          /* hash of cms fqdn */
   unsigned char   serverAuthKey[SERVER_AUTH_KEY_LEN];
} serverAuthKey_Info;





static serverAuthKey_Info serverAuthKey_table[MAX_KEY_STORAGE_ENTRIES];



/*
******************************************************************************
** FUNCTION:  secUpdateServerAuthKey
**
** PURPOSE:   Update sever auth key for this call agent
**
** PARAMETERS:
**    1. session key value
**    2. fqdn of call agent
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void secUpdateServerAuthKey(
      unsigned char   *session_key,
      unsigned long   session_key_len,
      unsigned char   *fqdn)
{
   int             i;
   unsigned long   tmp_len;
   unsigned char   tmp_hash[MD5_SIZE];          /* hash of cms fqdn */
   char            *p;
   unsigned char   lfqdn[255];

   strcpy(lfqdn, fqdn);
   p = lfqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, lfqdn, strlen(lfqdn),
         NULL, 0, tmp_hash, &tmp_len);

   /* See if an entry already exists that matches this fqdn */
   for (i=0;i<MAX_KEY_STORAGE_ENTRIES;i++)
   {
      if ( !memcmp(serverAuthKey_table[i].t_hash, tmp_hash, MD5_SIZE) )
      {
         SecLibGetCryptoIface()->secCryptoCalcHash(HASH_SHA1_TYPE, session_key, session_key_len,
               NULL, 0,
               (unsigned char *)(serverAuthKey_table[i].serverAuthKey),
               &tmp_len);
         memcpy(serverAuthKey_table[i].t_hash, tmp_hash, MD5_SIZE);
         return;
      }
   }

   /* find an open entry in the table */
   for (i=0;i<MAX_KEY_STORAGE_ENTRIES;i++)
      if ( serverAuthKey_table[i].t_hash[0] == '\0')
         break;
   if ( i == MAX_KEY_STORAGE_ENTRIES )
   {
      printf("*SEC PANIC* no more server auth key table entries.\n");
      return;
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_SHA1_TYPE, session_key, session_key_len,
         NULL, 0, (unsigned char *)(serverAuthKey_table[i].serverAuthKey),
         &tmp_len);
   memcpy(serverAuthKey_table[i].t_hash, tmp_hash, MD5_SIZE);
}


/*
******************************************************************************
** FUNCTION:  secGetServerAuthKey
**
** PURPOSE:   retreive server auth key
**
** PARAMETERS:
**    1. fqdn of call agent
**    2. server authentication key
**
**
** RETURNS:
**    index to the table if found,
**    0 otherwise.
**
** NOTE:
**     Caller responsible for allocating space for server authentication key
**
******************************************************************************
*/
unsigned long secGetServerAuthKey(
      unsigned char   *fqdn,
      unsigned char   *server_auth_key,
      unsigned long   *server_key_len)
{
   int              i;
   unsigned long    tmp_len;
   unsigned char    tmp_hash[MD5_SIZE];
   int              status;
   unsigned char    lfqdn[255];
   char             *p;

   strcpy(lfqdn, fqdn);
   p = lfqdn;
   while(*p)
   {
      *p++ = (char)tolower((int)*p);
   }

   SecLibGetCryptoIface()->secCryptoCalcHash(HASH_MD5_TYPE, lfqdn, strlen(lfqdn),
         NULL, 0, tmp_hash, &tmp_len);

   for (i=0;i<MAX_KEY_STORAGE_ENTRIES;i++)
   {
      if ( ! memcmp(serverAuthKey_table[i].t_hash, tmp_hash, MD5_SIZE))
      {
         status = SERVER_AUTH_KEY_FOUND;
         break;
      }
      else
      {
         status = SERVER_AUTH_KEY_NOT_FOUND;
      }
   }

   if ( i == MAX_KEY_STORAGE_ENTRIES)
      return status;

   if (status == SERVER_AUTH_KEY_FOUND)
   {
      memcpy(server_auth_key, serverAuthKey_table[i].serverAuthKey,
            SERVER_AUTH_KEY_LEN);
      *server_key_len = SERVER_AUTH_KEY_LEN;
      return status;
   }
   return SERVER_AUTH_KEY_NOT_FOUND;
}
