/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: event.c
*    Creation Date: 28 June 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:54:37 $
*
****************************************************************************
*    Description:
*
*     Broadcom Generic Operating System Functions. This file contains a VxWorks
*     specific implementation of event messages.
*
****************************************************************************/

#include <hausenv.h>
#include <string.h>
#include <bcmos.h>

/* Operating system specific includes */
#include <vxWorks.h>
#include "vxwrkdef.h"



/*****************************************************************************
** FUNCTION:   bcmOsEventCreate
**
** PURPOSE:    Creates a synchronization event object
**
** PARAMETERS: event [OUT] - handle for created event
**             taskId [IN[ - target task identifier. The created event will be sent
**                      to this task when bcmOsEventSend is invoked.
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventCreate( OS_EVENT *event, OS_TASKID *taskId )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventDelete
**
** PURPOSE:    Deletes a synchronization event object
**
** PARAMETERS: event [IN] - handle for event to be deleted
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventDelete( OS_EVENT *event )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetCreate
**
** PURPOSE:    Creates a synchronization event set object. Event sets are
**             a collection of event objects (which are created by bcmOsEventCreate)
**
** PARAMETERS: evtSet [OUT] - handle for created event set
**             taskId [IN] - target task identifier. Events contained within the set
**                      will be sent to this task when bcmOsEventSetSend is invoked.
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetCreate( OS_EVENTSET *evtSet, OS_TASKID *taskId )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetDelete
**
** PURPOSE:    Deletes a synchronization event set object
**
** PARAMETERS: event [IN] - handle for event set to be deleted
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetDelete( OS_EVENTSET *event )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetAdd
**
** PURPOSE:    Adds an event to an event set
**
** PARAMETERS: evtSet [IN] - handle for event set
**             event [IN] - the event which is to added to the event set
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetAdd( OS_EVENTSET *evtSet, OS_EVENT *event )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSend
**
** PURPOSE:    Sends an event to a task
**
** PARAMETERS: event [IN] - event to send
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSend( OS_EVENT *event )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetSend
**
** PURPOSE:    Sends a set of events to a task
**
** PARAMETERS: evtSet [IN] - event set to send
**
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetSend( OS_EVENTSET *evtSet )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventReceive
**
** PURPOSE:    Wait for an event to be sent to the calling task
**
** PARAMETERS: events [IN] - event to wait for
**             timeout - event timeout delay (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventReceive( OS_EVENT *event, OS_TICKS timeout )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetReceive
**
** PURPOSE:    Wait for a set of events to be sent to the calling task
**
** PARAMETERS: evtSet [IN] - events to wait for
**             waitForAll - if TRUE wait for all event to occur before returing,
**                          else only wait for a single event to occur
**             timeout - event timeout delay (in ticks)
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetReceive( OS_EVENTSET *evtSet, BOOL waitForAll, OS_TICKS timeout )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}


/*****************************************************************************
** FUNCTION:   bcmOsEventSetOccurred
**
** PURPOSE:    This function should be used by a task to determine if the specified
**             event has been triggered.  This is usually done immediately after a
**             call to bcmOsEventSetReceive() returns.
** PURPOSE:
**
** PARAMETERS: evtSet [IN] - event set handle
**             event [IN] - check is this event occurred
**             occurred [OUT] - TRUE if event occurred, else FALSE
**
** RETURNS:    OS_STATUS_OK - success
**             error code   - failure
**
** NOTE:
******************************************************************************/
OS_STATUS bcmOsEventSetOccurred( OS_EVENTSET *evtSet, OS_EVENT *event, BOOL *occurred )
{
   /* Not supported */
   return ( OS_STATUS_ERR );
}
