/* @(#) pSOSystem/MIPS V2.1.7: bsps/idt79465/bsp.h (&U&) 1.3.1.1 98/01/20 11:05:24 */
/***********************************************************************/
/*                                                                     */
/*   MODULE:  bsp.h                                                    */
/*   DATE:    98/01/20                                                 */
/*   PURPOSE: Defining characteristics of a board-support package      */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*           Copyright 1991 - 1996, Integrated Systems, Inc.           */
/*                      ALL RIGHTS RESERVED                            */
/*                                                                     */
/*   Permission is hereby granted to licensees of Integrated Systems,  */
/*   Inc. products to use or abstract this computer program for the    */
/*   sole purpose of implementing a product based on Integrated        */
/*   Systems, Inc. products.   No other rights to reproduce, use,      */
/*   or disseminate this computer program, whether in part or in       */
/*   whole, are granted.                                               */
/*                                                                     */
/*   Integrated Systems, Inc. makes no representation or warranties    */
/*   with respect to the performance of this computer program, and     */
/*   specifically disclaims any responsibility for any damages,        */
/*   special or consequential, connected with the use of this program. */
/*                                                                     */
/*---------------------------------------------------------------------*/
/*                                                                     */
/*   This file contains two sets of definitions:                       */
/*                                                                     */
/*     * The characteristics and capabilities of the board and its     */
/*       BSP, such as processor type, peripheral devices available,    */
/*       etc.                                                          */
/*                                                                     */
/*     * The vector numbers for the various interrupts and exceptions  */
/*       which can occur on the board.                                 */
/*                                                                     */
/***********************************************************************/
#if __cplusplus
extern "C" {
#endif

#ifndef _BSP_H
#define _BSP_H
				 
/*dg-mod added*/
#include <types.h>

#ifndef YES
#define YES 1
#endif
#ifndef NO
#define NO 0
#endif

/* Define BCM3300 codeset */
#ifndef BCM3300
#define BCM3300
#endif

/* Define this to use flash for NV storage instead of Xicor */
#define NV_USE_FLASH

/***********************************************************************/
/*                                                                     */
/*                                                                     */
/*      B O A R D - S U P P O R T   P A C K A G E   P R O F I L E      */
/*                                                                     */
/*                                                                     */
/***********************************************************************/
#define BSP_BCM93350
/*#define MIPS     5*/
#define FAMILY_MIPS     5

/*=====================================================================*/
/*                                                                     */
/*              B A S I C   C H A R A C T E R I S T I C S              */
/*                                                                     */
/*   * BSP_VERSION         - specifies revision level of pSOSystem     */
/*                           which this BSP was written to support     */
/*   * BSP_CPUFAMILY       - CPU processor FAMILY (68000, 88000,       */
/*                           80960, or 80386)                          */
/*   * BSP_CPU             - Specific processor within family (PPC603, */
/*                           PPC604, etc)                              */
/*   * BSP_FPU             - Floating-point unit (yes/no)              */
/*   * BSP_MMU             - Memory Management Unit (yes/no)           */
/*   * BSP_RAM_BASE        - Starting address of RAM as seen by CPU    */
/*   * BSP_VME             - YES if this is a VME board                */
/*   * BSP_LITTLE_ENDIAN   - YES if board is "little-endian"           */
/*                                                                     */
/*=====================================================================*/
#if defined(TARGETOS_vxWorks)
#else
#define BSP_VERSION            250
#endif
#define BSP_CPUFAMILY          FAMILY_MIPS
#define BSP_CPU                CPU_R4000    /* From pSOS's point of view, the */
														/* MR4001 CPU will be treated like */
														/* the R3051 CPU */

														/* The node configuration table */
                                          /* is set by reading the processor */
                                          /* version register. */
#define BSP_FPU                NO /*YES*/
#define BSP_MMU                NO
#define BSP_RAM_BASE           0x80000000  /*0xa0000000*/
#define BSP_VME                NO
#define BSP_LITTLE_ENDIAN      NO


/*=====================================================================*/
/*                                                                     */
/*            M I S C E L L A N E O U S   F E A T U R E S              */
/*                                                                     */
/*   * BSP_PARMS     - size in bytes of board-specific parameters      */
/*                     which should be stored in non-volatile RAM and  */
/*                     modified via the optional startup dialog.       */
/*                     0 = none.                                       */
/*   * BSP_ABORTSW   - Front panel "abort" button (YES/NO)             */
/*                                                                     */
/*=====================================================================*/
#define BSP_PARMS               18 

/*dg-mod, added NI selection (drv_conf will use this to determine which driver to load)*/
#define NI_SELECT_ETH                  1
#define NI_SELECT_USB                  2
#define NI_SELECT_PCI                  3
#define NI_SELECT_HPNA                 4

/* USB driver selection*/
#define USB_DRIVER_SELECT_NDIS         1
#define USB_DRIVER_SELECT_TEST         2

/* USB Vendor ID and Device ID for USB device*/
#define	USB_VENDOR_ID 	0x14e4         /* BRCM = 0x14e4,  Intel = 0x8086*/
#define	USB_DEVICE_ID 	0x3345         /* BRCM = 0x3345 */

#define BSP_ABORTSW            NO

/*=====================================================================*/
/*                                                                     */
/*                   S E R I A L   C H A N N E L S                     */
/*                                                                     */
/*   * BSP_SERIAL indicates the number of serial channels supported    */
/*     (0 = none)                                                      */
/*   * BSP_SERIAL_MINBAUD - lowest supported baud rate                 */
/*   * BSP_SERIAL_MAXBAUD - highest supported baud rate                */
/*                                                                     */
/*=====================================================================*/
#define BSP_SERIAL             2
#define BSP_SERIAL_MINBAUD     300 /*9600*/
#define BSP_SERIAL_MAXBAUD     115200

/* dg-mod, added */
#define BSP_NEW_SERIAL  YES

/*=====================================================================*/
/*                                                                     */
/*                    L A N   I N T E R F A C E S                      */
/*                                                                     */
/*   * BSP_LAN1            - YES if LAN is supported                   */
/*   * BSP_LAN1_MTU        - Maximum transmission unit (bytes)         */
/*   * BSP_LAN1_HWALEN     - Length of hardware address in bytes       */
/*   * BSP_LAN1_FLAGS      - Interface flags (per pNA manual)          */
/*        IFF_POLL             0x8000   interface is a polling type    */
/*        IFF_BROADCAST        0x0001   NI supports broadcasts         */
/*        IFF_RAWMEM           0x2000   driver uses "mblk" interface   */
/*   * BSP_LAN1_DPMEM      - YES if LAN has private memory             */
/*   * BSP_LAN1_NRRA       - No. of receive resource descriptors       */
/*   * BSP_LAN1_RRAMASK    - Index mask for circular wrapping          */
/*   * BSP_LAN1_NRBA       - No. of receive buffers (< NRRA)           */
/*   * BSP_LAN1_NRDA       - No. of receive descriptors                */
/*   * BSP_LAN1_NTDA       - No. of transmit descriptors               */
/*                                                                     */
/*=====================================================================*/
extern  void *                 NiLanHandlerPtr;   /* Ni entry point */

#define BSP_LAN1               NO //YES
#define BSP_LAN1_ENTRY         NiLan
#define BSP_LAN1_MTU           1500
#define BSP_LAN1_HWALEN        6
#define BSP_LAN1_FLAGS         0x2001 /*0xA001*/
#define BSP_LAN1_DPMEM         NO
#define BSP_LAN1_NRRA          32
#define BSP_LAN1_RRAMASK       (BSP_LAN1_NRRA - 1)
#define BSP_LAN1_NRBA          31
#define BSP_LAN1_NRDA          BSP_LAN1_NRBA
#define BSP_LAN1_NTDA          32

/*=====================================================================*/
/* Misc definitions                                                    */
/*=====================================================================*/

/*=====================================================================*/
/*                                                                     */
/*    S H A R E D   M E M O R Y   C O M M U N I C A T I O N            */
/*                                                                     */
/*   * BSP_SMEM            - Yes if shared memory communication is     */
/*                           supported                                 */
/*                                                                     */
/*=====================================================================*/
#define BSP_SMEM               NO

/*---------------------------------------------------------------------*/
/* MP_TAS determines queue lock implementation.  "HW" means the TAS    */
/* instruction is used.  Since not every board supports TAS, use "SW"  */
/* unless you are sure that all nodes can do TAS correctly.            */
/*---------------------------------------------------------------------*/
#define BSPS_MP_TAS       0         /* lock mode, 0 = SW, 1 = HW       */
 
/*---------------------------------------------------------------------*/
/* INT_MODE determines the method used to signal the arrival of        */
/* packets to a destination node.                                      */
/*---------------------------------------------------------------------*/
#define BSP_INT_MODE      1         /* 0 = Polling, 1 = Interrupt      */


/*=====================================================================*/
/*                                                                     */
/*                         S C S I                                     */
/*                                                                     */
/*=====================================================================*/
#define BSP_SCSI               NO

#define BSP_SCSI_TAPE          NO
/*=====================================================================*/
/*                                                                     */
/*              2 N D   T I M E R    F O R    p M O N T                */
/*                                                                     */
/*=====================================================================*/
#define BSP_TIMER2             NO

#define MBOX_CHECK_PT          NO  /*YES*/

#endif /* _BSP_H */

#if __cplusplus
}
#endif
