/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: globalCfg.h
*    Creation Date: 4 July 2000 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:54:23 $
*
****************************************************************************
*    Description:
*
*     Global default build options.  To override defaults in globalCfg.h
*     make a file called globalCfgCustom.h in your build directory.
*
****************************************************************************/

#ifndef GLOBAL_CFG_H
#define GLOBAL_CFG_H

#ifdef __cplusplus
extern "C" {
#endif

#include <hausenv.h>


/* Stand-alone MTA support */
#define GLOBAL_CFG_SMTA_SUPPORT        0

/* IPSEC Support is a MUST */
#define GLOBAL_CFG_IPSEC_SUPPORT       0


/* Maximum number of total endpoints. This MUST be set equal to the maximum number
 * of VOCM endpoints (VOCM_CFG_MAX_ENDPT) plus the maximum number of VOHN endpoints
 * (VOHN_CFG_MAX_ENDPT). vocmCfg.h and vohnCfg.h have been purposefully excluded
 * from the list of include files to avoid creating a dependency upon them.
 * i.e. If a VOCM build option is changed, we don't want this to cause every file
 * that includes cfgGlohal.h to re-compile. Hence, GLOBAL_CFG_MAX_ENDPT must
 * be set manually. */
#define GLOBAL_CFG_MAX_ENDPT           4


/* Maximum number of total connections. This MUST be set equal to the maximum number
 * of VOCM connections (VOCM_CFG_MAX_CNX) plus the maximum number of VOHN connections
 * (VOHN_CFG_MAX_CNX). vocmCfg.h and vohnCfg.h have been purposefully excluded
 * from the list of include files to avoid creating a dependency upon them.
 * i.e. If a VOCM build option is changed, we don't want this to cause every file
 * that includes cfgGlohal.h to re-compile. Hence, GLOBAL_CFG_MAX_CNX must
 * be set manually. */
#define GLOBAL_CFG_MAX_CNX             6



/* Service-flow options */
#if !WIN32
   /* NOT Win32 */

   #if GLOBAL_CFG_SMTA_SUPPORT
      /* S-MTA */

      /* 1 if the HIGH perforance UDP stack (serviceflow) should be used for RTP
       * transportation. 0 if operating system's best-effort UDP/IP stack should
       * be used */
      #define GLOBAL_CFG_SF_USED       0

   #else
      /* E-MTA */

      /* 1 if the HIGH perforance UDP stack (serviceflow) should be used for RTP
       * transportation. 0 if operating system's best-effort UDP/IP stack should
       * be used */
      #define GLOBAL_CFG_SF_USED       1

   #endif

#else
   /* WIN32 */

   /* Windows Sockets must be used to transport RTP data. Always set to 0. */
   #define GLOBAL_CFG_SF_USED          0

#endif



/*
*****************************************************************************
** CUSTOM CONFIGURATION - This must be included after all default
** build configuration options. Custom include files should undef default
** config options and then re-define them.
*****************************************************************************
*/
#include <globalCfgCustom.h>


#ifdef __cplusplus
    }
#endif

#endif /* GLOBAL_CFG_H */



