/***************************************************************************
*    Copyright 2000  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: haus_cpu.h
*    Creation Date: 31 May 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.1 $
*        $Date: 2002/08/21 17:54:24 $
*
****************************************************************************
*    Description:
*
*     Figures out which processor is being targeted.
*
*     This file uses macros from the compiler to determine which CPU
*     we're running under and generate an appropriate HAUS_CPU_xxx
*     macro.
*
****************************************************************************/

#if !defined( HAUS_CPU_H )
#define  HAUS_CPU_H

/*----- MIPS --------------------------------------------------------------*/

#if defined( __mips__ )

#define  HAUS_CPU_MIPS     1
#define  HAUS_CPU_STRING   "MIPS"

#if !defined( MIPS_CPU )
#define  MIPS_CPU    /* Deprecated - for backwards compatibility           */
#endif

/*----- x86 - PC (GNU or VC) -----------------------------------------------*/

#elif defined( _M_IX86 ) || defined( _X86 )

#define  HAUS_CPU_X86      1
#define  HAUS_CPU_STRING   "x86"

#if !defined( PC )
#define  PC           /* Deprecated - for backwards compatibility           */
#endif


/*----- Something we don't recognize ---------------------------------------*/

#else

#error haus_cpu.h: Unrecognized CPU

#endif

/*----- CPU ENDIAN-ness ---------------------------------------*/

#if HAUS_CPU_MIPS
   #define CPU_BIG_ENDIAN     1
   #define CPU_LITTLE_ENDIAN  0
#elif HAUS_CPU_X86
   #define CPU_BIG_ENDIAN     0
   #define CPU_LITTLE_ENDIAN  1
#else
   #error haus_cpu.h: Unrecognized CPU
#endif


/*-------------------------------------------------------------------------
* Now do some consistency checks
*/

#if defined( PC ) && !defined( HAUS_CPU_X86 )
#error "haus_cpu.h: PC is defined and HAUS_CPU_X86 isn't"
#endif

#if defined( MIPS_CPU ) && !defined( HAUS_CPU_MIPS )
#error "haus_cpu.h: MIPS_CPU is defined and HAUS_CPU_MIPS isn't"
#endif

#endif   /* HAUS_CPU_H  */

