/***************************************************************************
*    Copyright 2001  Broadcom Corporation
*    All Rights Reserved
*    No portions of this material may be reproduced in any form without the
*    written permission of:
*             Broadcom Corporation
*             16251 Laguna Canyon Road
*             Irvine, California  92618
*    All information contained in this document is Broadcom Corporation
*    company private, proprietary, and trade secret.
*
****************************************************************************
*
*    Filename: subkeyStore.h
*    Creation Date: 20 August 2001 (v0.00)
*    VSS Info:
*        $Revision: 1.2 $
*        $Date: 2005/02/17 21:13:34 $
*
****************************************************************************
*    Description:
*
*      Header file for Implementation of subkey storage and expiration 
*
****************************************************************************/

#ifndef SUBKEYSTORAGE_H
#define SUBKEYSTORAGE_H



#ifdef __cplusplus
extern "C"
{
#endif


#include <string.h>
#include <SecLib.h>
#include <ChKerbNamespace.h>

#define SUBKEY_LENGTH                 46


typedef enum
{
   SUBKEY_EXPIRED,
   SUBKEY_WITHIN_GRACE_PERIOD,
   SUBKEY_VALID,
};
   
/*
******************************************************************************
** FUNCTION:  secStoreSubkeyInfo 
**
** PURPOSE:   store subkey for expiry, endpoint sharing etc.
**
** PARAMETERS:
**    1. subkey value
**    2. subkey length
**    3. fqdn of call agent
**    4. security parameter lifetime - number of seconds fron now.
**    5. grace period - in seconds
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
void secStoreSubkeyInfo(
      unsigned char   *subkey,
      unsigned char   *fqdn,
      unsigned long   sec_params_lifetime,
      unsigned long   grace_period);


/*
******************************************************************************
** FUNCTION:  secRetreiveSubkeyInfo 
**
** PURPOSE:   retreive subkey for expiry, endpoint sharing etc.
**
** PARAMETERS:
**    1. subkey value
**    2. subkey length
**    3. fqdn of call agent
**    4. security parameter lifetime - number of seconds fron now.
**    5. grace period - in seconds
**
**
** RETURNS:
**    index to the table if found,
**    0 otherwise.
**
** NOTE:
**     Caller responsible for allocating space for subkey
**
******************************************************************************
*/
unsigned long secRetreiveSubkeyInfo(
      unsigned char   *subkey,
      unsigned char   *fqdn,
      unsigned long   *sec_params_lifetime,
      unsigned long   *grace_period,
	   int             *status);

 
/*
******************************************************************************
** FUNCTION:  secRemoveSubkeyStored 
**
** PURPOSE:   remove subkey for the fqdn entry
**
** PARAMETERS:
**    1. fqdn of call agent
**
**
** RETURNS:
**
** NOTE:
**
******************************************************************************
*/
unsigned long secRemoveSubkeyStored(
      unsigned char   *fqdn);


#ifdef __cplusplus
}
#endif

#endif
